;DEBUG		SET	1				; define to run in debug mode

Ver		EQU	1				; version
Rev		EQU	4				; revision


;**********************************************************************
;                                                                     *
;    Filename:  dual.asm                                              *
;    Date:      September 17, 2007                                    *
;    Version:   1.0                                                   *
;                                                                     *
;                                                                     *
;    Author:    Jim Drew                                              *
;    Company:   Xtreme Power Systems                                  *
;                                                                     *
;**********************************************************************
;
; NOTICE:
;
; THIS CODE IS COPYRIGHT (C) 2007-2008 BY XTREME POWER SYSTEMS, LLC.
; YOU MAY USE THIS CODE FOR ANY COMMERCIAL OR PUBLIC PRODUCT AS LONG
; AS CREDIT IS GIVEN TO XTREME POWER SYSTEMS FOR PROVIDING THE BASE
; CODE FOR YOUR PRODUCT.  THIS CODE MAY NOT BE USED IN ANY OTHER
; APPLICATION WITHOUT PERMISSION.
;
;
; This device is designed to communicate with the XtremeLink receiver.
; This particular unit is two sensors in one unit. One sensor detects
; incoming pulses for measuring RPMs, and the other sensor detects the
; voltage through a resistive sensor for measuring temperature.
;
; Input packets are those sent from the receiver to the sensor.
; Output packets are those sent from the sensor to the receiver.  The
; CRC is a 8 bit checksum calculated by adding all packet bytes
; together and XOR'ing with 0xFF.
;
; Notes:
;
; The serial lines Tx and Rx are tied together via a diode to make
; a 1-wire bus interface.  Because of this, any data sent to the Tx
; port will also appear on the Rx port.  It is necessary to get the
; byte of data from the Rx port after sending it to the Tx port.  If
; you don't, the receive FIFO will fill up after nearly 3 bytes and
; cause problems when trying to receive any incoming data that
; immediately follows.  If the SendByte routine fails, it means
; that the serial port is defective (not receiving data).
;
; The code handling the RPM count has been deliberately removed as
; this is an example code (skeleton) that is actually complete minus
; this code.  This sensor is available as a product, and such we did
; not feel that we should just give it away.  This code is about 95%
; of what you need to make a sensor.  In fact, you just need to come
; up with your own sensor handling code, everything else is here and
; ready to communicate with the XtremeLink telemetry system.  If you
; want just single sensor support, just remove the 2nd sensor code
; handling and you will be done.
;
; You can use the Developer Tool in the XDP.EXE program to send/receive
; packets to test your sensor, change settings, etc.
;
; Please direct any questions to our technical support department:
;
; tech@xtremepowersystems.net
;
;
;**********************************************************************
; Revision History
;
; v1.0 - 09/17/07
;
;----------------------------------------------------------------------
; v1.1 - 08/06/08
;
; Changed ReqsPerSec to TelRate.
;
;----------------------------------------------------------------------
; v1.2 - 08/12/08
;
; Fixed DDRC error (RCO - TempIn) was not being set to an input.
;
; Changed 0x80/0x81 to return after being called instead of jumping
; to the mainloop.
;
;----------------------------------------------------------------------
; v1.3 - 08/17/08
;
; Added new output type definitions to support GPS sensors.
;
;----------------------------------------------------------------------
; v1.4 - 08/23/08
;
; Moved serial buffer to high memory.
;
; Changed debounce routine for SW1 so that it actually works as a
; debounce and not a delay.



;**********************************************************************

; NOTE - RPM COUNTING ROUTINE REMOVED
;
; NO DATA IS RETURNED FOR RPM - PLEASE MAKE YOUR OWN CODE FOR THIS FUNCTION.
; THIS SENSOR IS A COMMERCIAL PRODUCT AND CAN BE PURCHASED FROM XTREME POWER
; SYSTEMS COMPLETE.



		list      P=PIC16F688			; list directive to define processor
		#include <p16f688.inc>
		__CONFIG _FCMEN_OFF & _IESO_OFF & _BOD_ON & _CPD_OFF & _CP_ON & _MCLRE_OFF & _PWRTE_ON & _WDT_OFF & _INTRC_OSC_NOCLKOUT

		#include <xps.inc>		 	; Xtreme Power Systems includes
		#include <sensors.inc>		 	; includes for sensors
		errorlevel	-302,-307,-306,-207	; suppress messages 302, 306, 307 & 207 from list file


;**********************************************************************
; PINOUT DEFINITION FOR XPS SENSOR 1 CPU (16F688)
;
;
;                            -------------------------
;                           |                         |
;                  (+5) ---1| Vss                 Vdd |14--- GND
;                           |                         |
;                   N/C ---2| RA5             RA0/AN0 |13--- N/C (PGC)
;                           |                         |
;                   N/C ---3| RA4/AN3         RA1/AN1 |12--- N/C (PGD)
;                           |                         |
;               SW1 (I) ---4| RA3/MCLR        RA2/AN2 |11--- (I) RPM Counter In
;                           |                         |
;      Serial Input (I) ---5| RC5/Rx          RC0/AN4 |10--- (A) Temperature Sensor Voltage In
;                           |                         |
;     Serial Output (O) ---6| RC4/Tx          RC1/AN5 |9 --- (O) LED1
;                           |                         |
;                   N/C ---7| RC3/AN7         RC2/AN6 |8 --- (O) LED2
;                           |                         |
;                            -------------------------


;**********************************************************************
; PORT DEFINITIONS

#DEFINE		Rsrvd1	PORTA,RA0			; no connection
#DEFINE		Rsrvd2	PORTA,RA1			; no connection
#DEFINE		RPM	PORTA,RA2			; RPM counter input
#DEFINE		SW1	PORTA,RA3			; switch input
#DEFINE		Rsrvd3	PORTA,RA4			; no connection
#DEFINE		DataOut	PORTA,RA5			; serial data output
DDRA		EQU	(1<<RA2)			; RA2 is an inputs, all others are outputs
PULA		EQU	(1<<RA2)			; pull up on RA2

#DEFINE		TempIn	PORTC,RC0			; temperature sensor input pin
#DEFINE		LED1	PORTC,RC1			; green LED output
#DEFINE		LED2	PORTC,RC2			; red LED output
#DEFINE		Rsrvd4	PORTC,RC3			; no connection
#DEFINE		TxOut	PORTC,RC4			; serial data out
#DEFINE		RxIn	PORTC,RC5			; serial data in
DDRC		EQU	(1<<RC5)|(1<<RC0)		; RC5/RC0 are inputs, all others are outputs

; change this for new PCB/CPU!!

ANALOG		EQU	(1<<ANS4)			; analog input (RC0)

		IFDEF	DEBUG
		MESSG	"In DEBUG mode!"
		ENDIF



;**********************************************************************
; EQUATES

XTAL_FREQ	EQU	8000000				; 8Mhz clock
Baud125000	EQU	(((10*XTAL_FREQ/(16*125000))+5)/10)-1


;**********************************************************************
; REGISTER FILE DEFINITIONS
;
; Bank 0 available: 20h-7Fh
; Bank 1 available: A0h-EFh, F0h-FFh (mirror of 70h-7Fh)
; Bank 2 available: 120h-16Fh, 170h-17F (mirror of 70h-7Fh)
; Bank 3 available: 1F0h-1FFh (mirror of 70h-7Fh)


; --- BANK 0 Registers ---

; main registers

SystemFlags	EQU	0x20				; system flags
SystemEvent	EQU	0x21				; system event flags
TempH		EQU	0x22				; temperature high
TempL		EQU	0x23				; temperarure low
RPMH		EQU	0x24				; high byte of RPM counter
RPML		EQU	0x25				; low byte of RPM counter
CRC		EQU	0x26				; CRC for packets
DataByte	EQU	0x27				; temp byte for serial data
ACC0		EQU	0x28
ACC1		EQU	0x29
ACC2		EQU	0x2A
ACC3		EQU	0x2B
TEMP0		EQU	0x2C
TEMP1		EQU	0x2D
TEMP2		EQU	0x2E
TEMP3		EQU	0x2F

; EEPROM copy

s1Flags		EQU	0x30
s1DevID		EQU	0x31
s1Class		EQU	0x32
s1TelRate	EQU	0x33
s1OutType	EQU	0x34
s1OutFormat	EQU	0x35
s1Setting1	EQU	0x36
s1Setting2	EQU	0x37

s2Flags		EQU	0x38
s2DevID		EQU	0x39
s2Class		EQU	0x3A
s2TelRate	EQU	0x3B
s2OutType	EQU	0x3C
s2OutFormat	EQU	0x3D
s2Setting1	EQU	0x3E
s2Setting2	EQU	0x3F

; Math package and misc. registers

NumH		EQU	0x60				; 16 bit number high byte
NumL		EQU	0x61				; 16 bit number low byte
TempLength	EQU	0x62
Delay1		EQU	0x63				; temp for delay counter
Delay2		EQU	0x64				; temp for delay counter
TenK		EQU	0x65
Thou		EQU	0x66
Hund		EQU	0x67
Tens		EQU	0x68
Ones		EQU	0x69

; Shared registers

ICD_RESERVED	EQU	0x70				; reserved for ICD
W_Temp		EQU	0x71				; variable used for context saving 
PCLATH_Temp	EQU	0x72
STATUS_Temp	EQU	0x73				; variable used for context saving
Scratch1	EQU	0x74				; scratch register
Scratch2	EQU	0x75				; ""
Scratch3	EQU	0x76				; ""
ADScratch	EQU	0x77				; scratch for A/D conversion routine
IntScratch	EQU	0x78				; scratch register 1 for interrupts
FlashScratch	EQU	0x79				; scratch register for flash routines
FLASHADDRH	EQU	0x7A				; high address byte for flash read/write
FLASHADDRL	EQU	0x7B				; low address byte for flash read/write
DATAH		EQU	0x7C				; high byte of read/write data word
DATAL		EQU	0x7D				; low byte of read/write data word

; 64 byte serial buffer (0xA0-0xDF)

Command		EQU	0xA0
CmdDevID	EQU	0xA1
CmdLength	EQU	0xA2
CmdData		EQU	0xA3

DataBuffer	EQU	0xE0				; temp storage


; SystemFlags bits

sf_SW1		EQU	0				; push button switch state, 0=switch down, 1=switch up


; SystemEvent flag bits


; Output types (byte definition)

ot_Blank	EQU	0				; nothing after data
ot_Voltage	EQU	1				; voltage output
ot_Current	EQU	2				; current output
ot_TempF	EQU	3				; temperature in F
ot_TempC	EQU	4				; temperature in C
ot_Percent	EQU	5				; percentage
ot_Up		EQU	6				; up arrow
ot_Down		EQU	7				; down arrow
ot_Left		EQU	8				; left arrow
ot_Right	EQU	9				; right arrow
ot_RPM		EQU	10				; revolutions per minute
ot_MPH		EQU	11				; miles per hour
ot_KPH		EQU	12				; kilometers per hour
ot_FPS		EQU	13				; feet per second
ot_MPS		EQU	14				; meters per second
ot_Knots	EQU	15				; knots
ot_Feet		EQU	16				; feet
ot_Meters	EQU	17				; meters
ot_Kilos	EQU	18				; kilometers
ot_Miles	EQU	19				; miles
ot_PSI		EQU	20				; PSI
ot_GForce	EQU	21				; G-force

; Output format (bit definitions)

of_Dec1		EQU	0				; 00=no decimal place, 01=1 decimal place
of_Dec2		EQU	1				; 10=2 decimal places, 11=3 decimal places
of_DataFormat1	EQU	2				; 00=HEX, 01=BCD
of_DataFormat2	EQU	3				; 10=ACSII, 11=reserved
of_PlaceType1	EQU	4				; 00=use period (.), 01=use colon (:)
of_PlaceType2	EQU	5				; 10=use comma (,), 11=use dash (-)
of_Reserved1	EQU	6				; reserved
of_Reserved2	EQU	7				; reserved



;**********************************************************************
; MACRO DEFINITIONS

LED_RED_ON	MACRO
		bsf	LED2
		ENDM

LED_RED_OFF	MACRO
		bcf	LED2
		ENDM

LED_GREEN_ON	MACRO
		bsf	LED1
		ENDM

LED_GREEN_OFF	MACRO
		bcf	LED1
		ENDM

LED_ORANGE_ON	MACRO
		bsf	LED1
		bsf	LED2
		ENDM

LED_OFF		MACRO
		bcf	LED1
		bcf	LED2
		ENDM

BANK0		MACRO
		bcf	STATUS,RP0			; select bank 0
		ENDM

BANK1		MACRO
		bsf	STATUS,RP0			; select bank 1
		ENDM

CLEAR_RAM	MACRO
		movlw	0x20				; start of RAM
		movwf	FSR				; store in indirect pointer
ClearRAMLoop	clrf	INDF				; clear the RAM location
		incf	FSR,f
		movf	FSR,w
		xorlw	0x080				; compare for 0x80 (end +1)
		btfss	STATUS,Z
		goto	ClearRAMLoop			; loop until done
		ENDM


;**********************************************************************
; Code starts here!

        	ORG     0x00				; code start address
		goto	Begin				; set PCLATH and go to beginning of program

		ORG	0x04				; interrupt vector location
		movwf	W_Temp				; save off current W register contents (saves can be in bank 0 or bank 1)
		swapf	STATUS,w			; swap status to be saved into W
		clrf	STATUS				; bank 0, regardless of current bank, clears IRP, PR1, RP0
		movwf	STATUS_Temp			; save status 
		movf	PCLATH,w
		movwf	PCLATH_Temp
		goto	IntHandler			; go to interrupt handler!


;**********************************************************************
; Initialize the PIC

Begin		BANK0					; select bank 0
		movlw	(1<<CM2)|(1<<CM1)|(1<<CM0)	; turn off comparators
		movwf	CMCON0
		movlw	0
		movwf	INTCON				; disable all interrupts
		movwf	PORTA
		movwf	PORTC

		BANK1					; select bank 1
;		movlw	(1<<IRCF2)|(1<<IRCF1)		; 4 Mhz clock
		movlw	(1<<IRCF2)|(1<<IRCF1)|(1<<IRCF0) ; 8 Mhz clock
		movwf	OSCCON	
		movlw	(1<<PS1)			; enable weak pull ups, external interupt on falling edge, TMR0 on internal clock, TMR0 prescaler, 1:8 prescaler
		movwf	OPTION_REG			; set option register
		movlw	ANALOG				; get analog inputs
		movwf	ANSEL				; set analog control reg
		movlw	(1<<ADCS2)|(1<<ADCS0)		; Fosc/16
		movwf	ADCON1
		movlw	DDRA				; data direction value
		movwf	TRISA				; setup PORT A I/O pins
		movlw	PULA				; pull-up bits
		movwf	WPUA				; set pull-ups
		movlw	DDRC				; data direction value
		movwf	TRISC				; setup PORT C I/O pins

		BANK0
		movlw	(1<<TXEN)			; enable tx with baud rate generator low
		movwf	TXSTA
		movlw	(1<<BRG16)			; enable 16 bit baud rate generator
		movwf	BAUDCTL
		movlw	Baud125000			; set baud rate to 125000
		movwf	SPBRG				; set low byte of 16 bit baud rate generator
		clrf	SPBRGH				; clear high byte of 16 bit baud rate generator
		movlw	(1<<SPEN)|(1<<CREN)		; enable serial port Rx/Tx and enable receive
		movwf	RCSTA

		CLEAR_RAM				; clear RAM locations 0x20-0x7F

		call	LOAD_EEPROM			; load the default EEPROM settings
		clrf	ADCON0				; disable A/D converter
;		bsf	INTCON,GIE			; turn on interrupts (global interrupt enable)

		call	Delay10ms			; make sure we are powered up and ready

		LED_RED_ON

;**********************************************************************
; Mainloop
;
; It is best in this application to use the pin change interrupt
; to count the RPM transitions.  Use TIMER1 as a clock source
; and count the changes per period to determine the RPM.

MainLoop	btfss	SW1				; look for switch (to set config settings if needed)
		goto	SW1Down

		call	GetPacket			; get packet data
		btfsc	STATUS,C			; was there a timeout?
		goto	MainLoop

		call	HandlePacket			; Check the device ID and handle packet if it matches

;----------------------------------------------------------------------
; Your code to change settings via push button and LED
; blinks would go here!

SW1Down		goto	MainLoop



;**********************************************************************
; A/D Input - Voltage
;
; RC0/AN4
;
; This routine reads the analog voltage on the temperature pin and
; stores the voltage value as 10 bits in TempH and TempL.
;
; Note: the voltage is pulled high via a biasing resistor that must
; be properly selected to match the resistance of the thermal sensor.

CheckTemp	movlw	(1<<ADFM)|(1<<CHS2)|(1<<ADON)	; Channel 4 (AN4), A/D converter is on with right justify
		movwf	ADCON0				; enable a/d hardware
		movlw	50
		movwf	ADScratch
DoConvert1	decfsz	ADScratch,f			; delay for A/D setup
		goto	DoConvert1

		bsf	ADCON0,GO			; start conversion
WaitConvert1	btfsc	ADCON0,GO			; wait for conversion to finish
		goto	WaitConvert1

		movf	ADRESH,w			; get result high byte from A/D
		movwf	TempH				; save voltage high byte
		BANK1
		movf	ADRESL,w
		BANK0
		movwf	TempL				; save voltage low byte
		bcf	ADCON0,ADON			; turn off A/D hardware (saves power)
		return


;**********************************************************************
; This routine sends a byte of data and waits for that byte to be
; echo'd back to the Rx port.
;
; w = byte to send

SendByte	movwf	DataByte			; save byte to send
		movlw	195				; time set for 248us
		movwf	TMR0				; reset TMR0
		bcf	INTCON,T0IF			; clear any pending interrupt flags

SendByteLoop	btfsc	INTCON,T0IF			; did TMR0 wrap?
		goto	TMR0Expired			; yes, so do a set return value

		btfss	PIR1,TXIF			; wait for tx buffer to empty
		goto	SendByteLoop

		movf	DataByte,w			; get byte to send
		movwf	TXREG				; send byte

; routine falls through to fetch the byte from
; the receive port


;**********************************************************************
; This routine gets a byte of data from the serial port.
;
; A much better method would be to use the interrupt handler and setup
; a circular buffer.  We will do that at some point as this will make
; polling of the input sensor much easier to handle.
;
; w > byte
; C = clear for good byte, set for no byte

GetByte		movlw	195				; time set for 248us
		movwf	TMR0				; reset TMR0
		bcf	INTCON,T0IF			; clear any pending interrupt flags

		btfsc	RCSTA,FERR			; was there a framing error?
		movf	RCREG,w				; reading RCREG clears FERR flag

		btfsc	RCSTA,OERR			; was there an overflow error?
		goto	OverrunError			; no, so no need to reset serial port

GetByteLoop	btfsc	INTCON,T0IF			; did TMR0 wrap?
		goto	TMR0Expired			; yes, so do a set return value

		btfss	PIR1,RCIF			; serial byte ready?
		goto	GetByteLoop

		movf	RCREG,w				; get byte
		bcf	STATUS,C			; clear carry flag
		return

TMR0Expired	bsf	STATUS,C			; set carry flag to indicate no data returned
		return

OverrunError	bcf	RCSTA,CREN			; turn CREN off
		movf	RCREG,w				; flush all 3
		movf	RCREG,w				; fifo bytes
		movf	RCREG,w				; before turning CREN back on
		bsf	RCSTA,CREN	
		goto	GetByteLoop


;**********************************************************************
; This routine gets a packet from the serial port.  If a serial byte
; is not available, or the CRC fails, the carry flag is set.  Otherwise,
; the carry flag is cleared to indicate a successful packet was obtained.
;
; The packet data is stored in the SerialBuffer

GetPacket	movlw	Command				; start point for data
		movwf	FSR				; set index
		clrf	CRC				; clear checksum

		call	GetByte				; get command byte from serial port
		btfsc	STATUS,C			; carry clear?
		return					; no, so return w/carry set

		addwf	CRC,f				; add to checksum
		movwf	INDF				; save command
		incf	FSR,f				; inc index

		call	GetByte				; get device ID byte from serial port
		btfsc	STATUS,C			; carry clear?
		return					; no, so return w/carry set

		addwf	CRC,f				; add to checksum
		movwf	INDF				; save device ID
		incf	FSR,f				; inc index

		call	GetByte				; get length byte from serial port
		btfsc	STATUS,C			; carry clear?
		return					; no, so return w/carry set

		addwf	CRC,f				; add to checksum
		movwf	INDF				; save CmdLength
		addlw	1				; add one to length for decf
		movwf	TempLength			; save command length
		addlw	-61				; see if the value is >30
		btfsc	STATUS,C
		goto	LenError

		incf	FSR,f				; inc index
		goto	GetCmdData

GetCmdLoop	call	GetByte				; get byte from serial port
		btfsc	STATUS,C			; carry clear?
		return					; no, so return w/carry set

		addwf	CRC,f				; add to checksum
		movwf	INDF				; store byte
		incf	FSR,f				; inc index
GetCmdData	decfsz	TempLength,f
		goto	GetCmdLoop

		call	GetByte				; get CRC byte from serial port
		btfsc	STATUS,C			; carry clear?
		return					; no, so return w/carry set

		xorlw	0xFF				; invert CRC
		xorwf	CRC,w				; compare to stored CRC
		bcf	STATUS,C			; clear carry
		btfss	STATUS,Z			; skip setting carry if CRC is correct
LenError	bsf	STATUS,C			; set carry
		return


;**********************************************************************
; This routine figures out the incomming packet and
; jumps to the appropriate routine.
		
HandlePacket	BANK1
		movf	Command,w			; get command from BANK1
		BANK0
		xorlw	0x80				; is command INITIALIZE?
		btfsc	STATUS,Z
		goto	cmdInitMode			; yes, so handle initialization

		xorlw	0x82 ^ 0x80 			; is command Get Data?
		btfsc	STATUS,Z
		goto	cmdGetData			; yes, handle sending data

		xorlw	0x83 ^ 0x82 			; is command Get Info?
		btfsc	STATUS,Z
		goto	cmdGetInfo			; yes, handle sending info

		xorlw	0x84 ^ 0x83 			; is command Get custom settings?
		btfsc	STATUS,Z
		goto	cmdGetSettings			; yes, handle sending config

		xorlw	0x85 ^ 0x84 			; is command set custom settings?
		btfsc	STATUS,Z
		goto	cmdSetSettings			; yes, handle sending config

		return


;**********************************************************************
; SENSOR PACKETS
;
; Init Mode
;
; This command puts the sensor into initialization mode, required to
; obtain a device ID from the receiver.  The device ID is used when
; communicating.
;
; Input: Command (0x80), Device ID (0xFF), Length (0), Checksum (CRC)
; Output: N/A

cmdInitMode	BANK1
		movf	CmdDevID,w			; make sure device ID is 0xFF
		BANK0
		xorlw	0xFF
		btfss	STATUS,Z
		return					; exit if not

RestartInit	LED_ORANGE_ON

; switch must be down for at least 100ms

WaitOnSW1	movlw	131
		movwf	Scratch1
		movlw	246
WaitDelay1	btfsc	SW1				; wait for switch to be down
		goto	WaitOnSW1

		addlw	1
		btfss	STATUS,Z
		goto	WaitDelay1

		decfsz	Scratch1,f
		goto	WaitDelay1

		LED_RED_OFF

; ok, switch is down so send 'Request Device ID' to the receiver.
;
; Output: Command (0x41),  Device ID (0xFF), Length (1), Class, Checksum (CRC)

		clrf	CRC				; clear checksum
		movlw	0x41				; command "Send ID Request"
		addwf	CRC,f				; add to checksum
		call	SendByte			; send command
		btfsc	STATUS,C
		goto	RestartInit

		movlw	0xFF				; device ID set to broadcast
		addwf	CRC,f				; add to checksum
		call	SendByte			; send device ID
		btfsc	STATUS,C
		goto	RestartInit

		movlw	1				; number of bytes we are sending (1 for class)
		addwf	CRC,f				; add to checksum
		call	SendByte			; send command length
		btfsc	STATUS,C
		goto	RestartInit

		movf	s1Class,w			; get sensor 1's type of device from RAM
		addwf	CRC,f				; add to checksum
		call	SendByte			; send device type
		btfsc	STATUS,C
		goto	RestartInit

		movf	CRC,w				; get CRC
		xorlw	0xFF				; invert it
		call	SendByte
		btfsc	STATUS,C
		goto	RestartInit

; packet sent, now we wait on the packet to be responded to
;
; Input: Command (0x81), Device ID (DevID), Length (1), # Requests per second, CRC

L1		call	GetPacket			; get packet (should be Device ID)
		btfsc	STATUS,C
		goto	RestartInit

		BANK1
		movf	Command,w			; get command from BANK1
		BANK0
		xorlw	0x81				; compare command to "Assign Device ID"
		btfss	STATUS,Z
		goto	L1				; retry getting packet just in case we were off a byte in the FIFO

		BANK1
		movf	CmdLength,w			; get command length
		BANK0
		xorlw	1				; is length set to 1?
		btfss	STATUS,Z
		goto	RestartInit			; no, so retry

		BANK1
		movf	CmdDevID,w			; get device ID
		BANK0
		movwf	DATAL				; set value for EEPROM write
		movlw	ee_s1DevID			; offset to sensor 1 device ID
		movwf	FLASHADDRL			; set address for EEPROM write
		call	EEPROM_WRITE			; save value in EEPROM

		BANK1
		movf	CmdData,w			; get 1st byte of data (requests per second)
		BANK0
		movwf	DATAL				; set value for EEPROM write
		movlw	ee_s1TelRate			; offset to sensor 1 update rate
		movwf	FLASHADDRL			; set address for EEPROM write
		call	EEPROM_WRITE			; save value in EEPROM

		LED_GREEN_OFF
		LED_RED_ON

		call	Delay250ms			; we need to wait for the receiver to store its data

; ok, sensor 1 is done so send Device ID request to the receiver for sensor 2.
;
; Output: Command (0x41),  Device ID (0xFF), Length (1), Class, Checksum (CRC)

		clrf	CRC				; clear checksum
		movlw	0x41				; command "Send ID Request"
		addwf	CRC,f				; add to checksum
		call	SendByte			; send number of bytes returned
		btfsc	STATUS,C
		goto	RestartInit

		movlw	0xFF				; device ID set to broadcast
		addwf	CRC,f				; add to checksum
		call	SendByte			; send device ID
		btfsc	STATUS,C
		goto	RestartInit

		movlw	1				; number of bytes we are sending (1 for class)
		addwf	CRC,f				; add to checksum
		call	SendByte			; send command length
		btfsc	STATUS,C
		goto	RestartInit

		movf	s2Class,w			; get sensor 2's type of device from RAM
		addwf	CRC,f				; add to checksum
		call	SendByte			; send device type
		btfsc	STATUS,C
		goto	RestartInit

		movf	CRC,w				; get CRC
		xorlw	0xFF				; invert it
		call	SendByte
		btfsc	STATUS,C
		goto	RestartInit

; packet sent, now we wait on the packet to be responded to
;
; Input: Command (0x81), Device ID (DevID), Length (1), # Requests per second, CRC

L2		call	GetPacket
		btfsc	STATUS,C
		goto	RestartInit

		BANK1
		movf	Command,w			; get command
		BANK0
		xorlw	0x81				; compare command to "Assign Device ID"
		btfss	STATUS,Z
		goto	L2				; retry getting packet just in case we were off a byte in the FIFO

		BANK1
		movf	CmdLength,w			; get command length
		BANK0
		xorlw	1				; is length set to 1?
		btfss	STATUS,Z
		goto	RestartInit			; no, so retry

		BANK1
		movf	CmdDevID,w			; get device ID
		BANK0
		movwf	DATAL				; set value for EEPROM write
		movlw	ee_s2DevID			; offset to sensor 2 device ID
		movwf	FLASHADDRL			; set address for EEPROM write
		call	EEPROM_WRITE			; save value in EEPROM

		BANK1
		movf	CmdData,w			; get 1st byte of data (requests per second)
		BANK0
		movwf	DATAL				; set value for EEPROM write
		movlw	ee_s2TelRate			; offset to sensor 2 update rate
		movwf	FLASHADDRL			; set address for EEPROM write
		call	EEPROM_WRITE			; save value in EEPROM

		LED_RED_OFF				; turn off green portion of LED to indicate sensor 2 complete
		goto	LOAD_EEPROM			; reload the EEPROM to setup new variables and return


;----------------------------------------------------------------------
; 0x82 - Get Data (input packet)
;
; This packet requests the sensor's data to be sent.
;
; Input: Command (0x82), Device ID (DevID), Length (0), CRC
; Output: Command (0x42), Device ID (DevID), Length (3+X), Class, Output Type, Output Format, (X) Data Bytes, CRC

cmdGetData	BANK1
		movf	CmdDevID,w			; get device ID
		BANK0
		xorwf	s2DevID,w			; check to see if it is sensor 2
		btfsc	STATUS,Z			; skip if not
		goto	GetData2			; else get sensor 2's data

		bsf	STATUS,C			; set carry in preparation for an error
		BANK1
		movf	CmdDevID,w			; get device ID again
		BANK0
		xorwf	s1DevID,w			; is it sensor 1's ID?
		btfss	STATUS,Z			; skip if it is
		return					; return with carry flag set (xor does not affect carry!)

GetData1	;call	CheckRPM
		LED_OFF
		clrf	CRC				; clear checksum
		movlw	0x42				; command "Send Data"
		addwf	CRC,f				; add to checksum
		call	SendByte			; send command
		BANK1
		movf	CmdDevID,w			; get requested device's ID
		BANK0
		addwf	CRC,f
		call	SendByte			; send Device ID
		movlw	3+2				; number of bytes we are sending (3+2, RPM high and low)
		addwf	CRC,f				; add to checksum
		call	SendByte			; send command length
		movf	s1Class,w			; get class
		addwf	CRC,f				; add to checksum
		call	SendByte			; send command length
		movf	s1OutType,w			; get output type
		addwf	CRC,f				; add to checksum
		call	SendByte			; send device type
		movf	s1OutFormat,w			; get output format
		addwf	CRC,f				; add to checksum
		call	SendByte			; send number of bytes returned
		movf	RPMH,w				; get RPM high
		addwf	CRC,f				; add to checksum
		call	SendByte			; send device type
		movf	RPML,w				; get RPM low
		addwf	CRC,f				; add to checksum
		call	SendByte			; send number of bytes returned
		movf	CRC,w				; get CRC
		xorlw	0xFF				; invert it
		call	SendByte
		bcf	STATUS,C
		LED_GREEN_ON
		return

GetData2	
		LED_OFF
		call	CheckTemp
		clrf	CRC				; clear checksum
		movlw	0x42				; command "Send Data"
		addwf	CRC,f				; add to checksum
		call	SendByte			; send command
		BANK1
		movf	CmdDevID,w			; get requested device's ID
		BANK0
		addwf	CRC,f
		call	SendByte			; send Device ID
		movlw	3+2				; number of bytes we are sending (3+2, Temp high and low)
		addwf	CRC,f				; add to checksum
		call	SendByte			; send command length
		movf	s2Class,w			; get class
		addwf	CRC,f				; add to checksum
		call	SendByte			; send command length
		movf	s2OutType,w			; get output type
		addwf	CRC,f				; add to checksum
		call	SendByte			; send device type
		movf	s2OutFormat,w			; get output format
		addwf	CRC,f				; add to checksum
		call	SendByte			; send number of bytes returned
		movf	TempH,w				; get current high
		addwf	CRC,f				; add to checksum
		call	SendByte			; send device type
		movf	TempL,w				; get current low
		addwf	CRC,f				; add to checksum
		call	SendByte			; send number of bytes returned
		movf	CRC,w				; get CRC
		xorlw	0xFF				; invert it
		call	SendByte
		bcf	STATUS,C
		LED_GREEN_ON
		return


;----------------------------------------------------------------------
; 0x83 - Get Info (input packet)
;
; This packet requests the information about the sensor (typically
; the vendor ID string).
;
; Input: Command (0x83), Device ID (DevID), Length (0), CRC
; Output: Command (0x43), Device ID (DevID), Length (Len of strings), String,CRC

cmdGetInfo	movlw	ee_s2Vendor
		movwf	FLASHADDRL			; set EEPROM address
		BANK1
		movf	CmdDevID,w			; get device ID
		BANK0
		xorwf	s2DevID,w			; check to see if it is sensor 2
		btfsc	STATUS,Z			; skip if not
		goto	SendVendorStr

		movlw	ee_s1Vendor
		movwf	FLASHADDRL			; set EEPROM address
		bsf	STATUS,C			; set carry in preparation for an error
		BANK1
		movf	CmdDevID,w			; get device ID again
		BANK0
		xorwf	s1DevID,w			; is it sensor 1's ID?
		btfss	STATUS,Z			; skip if it is
		return					; return with carry flag set (xor does not affect carry!)

SendVendorStr
		LED_OFF
		clrf	CRC				; clear checksum
		movlw	0x43				; command "Send Info"
		addwf	CRC,f				; add to checksum
		call	SendByte			; send command
		BANK1
		movf	CmdDevID,w			; send device ID
		BANK0
		addwf	CRC,f
		call	SendByte			; send Device ID
		call	EEPROM_READ			; get length from EEPROM
		movf	DATAL,w
		movwf	Scratch1			; use scratch 1 for length
		addwf	CRC,f				; add to checksum
		call	SendByte			; send byte
		incf	FLASHADDRL,f			; inc EEPROM address
GetInfoLoop	call	EEPROM_READ
		movf	DATAL,w				; get byte from EEPROM
		addwf	CRC,f				; add to checksum
		call	SendByte			; send byte
		incf	FLASHADDRL,f			; inc EEPROM address
		decfsz	Scratch1,f			; dec counter
		goto	GetInfoLoop			; loop until done

		movf	CRC,w				; get CRC
		xorlw	0xFF				; invert it
		call	SendByte
		bcf	STATUS,C			; clear carry (no error)
		LED_GREEN_ON
		return


;----------------------------------------------------------------------
; 0x84 - Request Sensor Settings (input packet)
;
; This packet requests the sensor's custom settings.
;
; Input: Command (0x84), Device ID (DevID), Length (0), CRC
; Output: Command (0x44), Device ID (DevID), Length (2), Setting 1, Setting 2, CRC

cmdGetSettings	BANK1
		movf	CmdDevID,w			; get device ID
		BANK0
		xorwf	s2DevID,w			; check to see if it is sensor 2 is being called
		btfsc	STATUS,Z			; skip if not
		goto	GetSettings2			; else sensor 2 is being called

		bsf	STATUS,C			; set carry in preparation for an error
		BANK1
		movf	CmdDevID,w			; get device ID again
		BANK0
		xorwf	s1DevID,w			; is it sensor 1's ID?
		btfss	STATUS,Z			; skip if it is
		return					; return with carry flag set (xor does not affect carry!)

GetSettings1
		LED_OFF
		clrf	CRC				; clear checksum
		movlw	0x44				; command "Send Config"
		addwf	CRC,f				; add to checksum
		call	SendByte			; send command
		BANK1
		movf	CmdDevID,w			; get requested device's ID
		BANK0
		addwf	CRC,f
		call	SendByte			; send Device ID

		movlw	2				; number of bytes we are sending (2, setting 1 and 2)
		addwf	CRC,f				; add to checksum
		call	SendByte			; send command length
		movf	s1Setting1,w			; get setting 1
		addwf	CRC,f				; add to checksum
		call	SendByte			; send device type
		movf	s1Setting2,w			; get setting 2
		addwf	CRC,f				; add to checksum
		call	SendByte			; send number of bytes returned
		movf	CRC,w				; get CRC
		xorlw	0xFF				; invert it
		call	SendByte
		bcf	STATUS,C
		LED_GREEN_ON
		return


GetSettings2
		LED_OFF
		clrf	CRC				; clear checksum
		movlw	0x44				; command "Send Config"
		addwf	CRC,f				; add to checksum
		call	SendByte			; send command
		BANK1
		movf	CmdDevID,w			; get requested device's ID
		BANK0
		addwf	CRC,f
		call	SendByte			; send Device ID
		movlw	2				; number of bytes we are sending (2, setting 1 and 2)
		addwf	CRC,f				; add to checksum
		call	SendByte			; send command length
		movf	s2Setting1,w			; get setting 1
		addwf	CRC,f				; add to checksum
		call	SendByte			; send device type
		movf	s2Setting2,w			; get setting 2
		addwf	CRC,f				; add to checksum
		call	SendByte			; send number of bytes returned
		movf	CRC,w				; get CRC
		xorlw	0xFF				; invert it
		call	SendByte
		bcf	STATUS,C
		LED_GREEN_ON
		return


;----------------------------------------------------------------------
; 0x85 - Set Sensor Settings (input packet)
;
; This packet sets the sensor's custom settings.
;
; Input: Command (0x85), Device ID (DevID), Length (2), Setting 1, Setting 2, CRC
; Output: None

cmdSetSettings	BANK1
		movf	CmdDevID,w			; get device ID
		BANK0
		xorwf	s2DevID,w			; check to see if it is sensor 2 is being called
		btfsc	STATUS,Z			; skip if not
		goto	SetSettings2			; else sensor 2 is being called

		bsf	STATUS,C			; set carry in preparation for an error
		BANK1
		movf	CmdDevID,w			; get device ID again
		BANK0
		xorwf	s1DevID,w			; is it sensor 1's ID?
		btfss	STATUS,Z			; skip if it is
		return					; return with carry flag set (xor does not affect carry!)

SetSettings1
		LED_OFF
		movlw	ee_s1Setting1
		movwf	FLASHADDRL			; set EEPROM address
		BANK1
		movf	CmdData,w
		BANK0
		movwf	s1Setting1			; set setting 1
		movwf	DATAL
		call	EEPROM_WRITE

		movlw	ee_s1Setting2
		movwf	FLASHADDRL			; set EEPROM address
		BANK1
		movf	CmdData+1,w
		BANK0
		movwf	s1Setting2			; set setting 2
		movwf	DATAL
		call	EEPROM_WRITE
		bcf	STATUS,C
		LED_GREEN_ON
		return


SetSettings2
		LED_OFF
		movlw	ee_s2Setting1
		movwf	FLASHADDRL			; set EEPROM address
		BANK1
		movf	CmdData,w
		BANK0
		movwf	s2Setting1			; set setting 1
		movwf	DATAL
		call	EEPROM_WRITE

		movlw	ee_s2Setting2
		movwf	FLASHADDRL			; set EEPROM address
		BANK1
		movf	CmdData+1,w
		BANK0
		movwf	s2Setting2			; set setting 2
		movwf	DATAL
		call	EEPROM_WRITE
		bcf	STATUS,C
		LED_GREEN_ON
		return


;----------------------------------------------------------------------
; Delays based on 8Mhz clock

Delay10ms	movlw	1
		goto	DelayCheater

Delay50ms	movlw	5				; 5 * 10ms = 50ms
		goto	DelayCheater

Delay100ms	movlw	10				; 10 * 10ms = 100ms
		goto	DelayCheater

Delay250ms	movlw	25				; 25 * 10ms = 250ms
		goto	DelayCheater

Delay500ms	movlw	50				; 50 * 10ms = 500ms
		goto	DelayCheater

Delay1000ms	movlw	100				; 100 * 10ms = 1000ms
DelayCheater	movwf	Delay1
OneSecLoop	call	DoDelay
		decfsz	Delay1,f
		goto	OneSecLoop

		return

; Delay 10ms based on 8Mhz clock (time excludes CALL and RETURN)

DoDelay		movlw	15+1				; 15*256=19200
		movwf	Delay2
		movlw	256-158				; (20000-19200)/5 = 160 (-2 for overhead)

Domsdelay	addlw	1
		btfsc	STATUS,Z
		decfsz	Delay2,f
		goto	Domsdelay

		nop
		return


;----------------------------------------------------------------------
; Debug routine to save packet information

SavePacket	movlw	0x80
		movwf	FLASHADDRL
		movlw	Command
		movwf	FSR
SavePacketLoop	movf	INDF,w
		movwf	DATAL
		call	EEPROM_WRITE
		incf	FLASHADDRL,f
		incf	FSR,f
		decfsz	Scratch3,f
		goto	SavePacketLoop

self		goto	self


;**********************************************************************
; EEPROM Control Routines
;
;
;----------------------------------------------------------------------
; EEPROM WRITE
;
; FLASHADDRL = address to write (0-127)
; DATAL = data to write

EEPROM_WRITE	movf	INTCON,w
		movwf	FlashScratch			; save interupt reg contents
		bsf	STATUS,RP0			; select BANK 1
		movf	FLASHADDRL,w			; get address to write
		movwf	EEADR				; store low address
		movf	DATAL,w				; get byte to write
		movwf	EEDATA				; store it
		bsf	EECON1,WREN			; do a WRITE function
		bcf	INTCON,GIE			; disable interrupts!
		movlw	0x55				; setup special write
		movwf	EECON2				; enable sequence
		movlw	0xAA
		movwf	EECON2
		bsf	EECON1,WR			; start write
		nop
		nop
WriteLoop	btfsc	EECON1,WR			; is write complete?
		goto	WriteLoop			; loop until done

		bcf	EECON1,WREN			; disable write capability
		bcf	STATUS,RP0			; select BANK 0
		btfsc	FlashScratch,GIE		; skip if GIE bit was clear
		bsf	INTCON,GIE			; restore GIE bit
		return


;----------------------------------------------------------------------
; EEPROM READ
;
; FLASHADDRL = address to read (0-127)
; DATAL > data read

EEPROM_READ	bsf	STATUS,RP0			; select bank 1
		movf	FLASHADDRL,w			; get address to read
		movwf	EEADR				; store low address
		bsf	EECON1,RD			; do a READ function
		nop
		nop
		movf	EEDATA,w			; get byte
		movwf	DATAL				; save it
		bcf	STATUS,RP0			; select BANK 0
		return


;----------------------------------------------------------------------
; This routines reads (loads) the EEPROM data and copies each byte to
; the proper RAM based location.

LOAD_EEPROM	clrf	FLASHADDRL			; start a EEPROM byte 0
		movlw	0x30				; set indirect pointer to be start of EEPROM data in RAM
		movwf	FSR				; set index pointer
_LoadEELoop	call	EEPROM_READ			; do read
		movf	DATAL,w				; get byte from temp storage
		movwf	INDF				; store indirectly
		incf	FSR,f				; increment indirect index pointer
		incf	FLASHADDRL,f			; increment address pointer
		movf	FLASHADDRL,w			; get address pointer
		sublw	16				; subtract 16 (2 sensors, each with 8 bytes)
		btfss	STATUS,Z			; is it equal?
		goto	_LoadEELoop			; nope, continue!

		return


;**********************************************************************
;
;                         INTERRUPT HANDLER
;
;**********************************************************************

IntHandler	btfsc	INTCON,INTF			; RA2/INT changed from HIGH to LOW?
		goto	DataInInt			; yes, handle data input interrupt

IntHandlerExit	movf	PCLATH_Temp,w			; retrieve copy of PCLATH register
		movwf	PCLATH
		swapf	STATUS_Temp,w			; retrieve copy of STATUS register
		movwf	STATUS				; restore pre-isr STATUS register contents
		swapf	W_Temp,f
		swapf	W_Temp,w			; restore pre-isr W register contents
		retfie					; return from interrupt


;**********************************************************************
; This is the interrupt handler

DataInInt	bcf	INTCON,INTF			; clear external interrupt flag
		goto	IntHandlerExit			; go to exit routine


;**********************************************************************
; EEPROM DATA CONFIGURATION
;
; There are 256 bytes of EEPROM that can be read/written via the
; EEREAD and EEWRITE routines.  The data structure is as follows:
;
; Byte(s)	Description
;----------------------------------------------------------------------
; 
; Sensor 1 determines the motor RPM
;
; 0x00		Sensor 1 flags for various states.
; 0x01		Sensor 1 ID
; 0x02		Sensor 1 Class
; 0x03		Sensor 1 telemetry update rate
; 0x04		Sensor 1 output type
; 0x05		Sensor 1 output format
; 0x06		Sensor 1 setting 1
; 0x07		Sensor 1 setting 2
;
; Sensor 2 determines the motor temperature
;
; 0x08		Sensor 2 flags for various states.
; 0x09		Sensor 2 ID
; 0x0A		Sensor 2 Class
; 0x0B		Sensor 2 telemetry update rate
; 0x0C		Sensor 2 output type
; 0x0D		Sensor 2 output format
; 0x0E		Sensor 2 setting 1
; 0x0F		Sensor 2 setting 2
;
; 0x10-0x7F	Reserved
; 0x80-0xBF	Sensor 1 vendor string
; 0xC0-0xFF	Sesnor 2 vendor string

ee_s1Flags	EQU	0
ee_s1DevID	EQU	1
ee_s1Class	EQU	2
ee_s1TelRate	EQU	3
ee_s1OutType	EQU	4
ee_s1OutFormat	EQU	5
ee_s1Setting1	EQU	6
ee_s1Setting2	EQU	7

ee_s2Flags	EQU	8
ee_s2DevID	EQU	9
ee_s2Class	EQU	10
ee_s2TelRate	EQU	11
ee_s2OutType	EQU	12
ee_s2OutFormat	EQU	13
ee_s2Setting1	EQU	14
ee_s2Setting2	EQU	15

ee_s1Vendor	EQU	0x80
ee_s2Vendor	EQU	0xC0

		ORG	0x2100

MyEEPROM	de	0				; 00 - Sensor 1 flags
		de	0				; 01 - Sensor 1 ID
		de	tc_rMotor1			; 02 - Sensor 1 Class (RPM)
		de	0				; 03 - Sensor 1 requests per second
		de	ot_RPM				; 04 - Sensor 1 displays as RPM
		de	(1<<of_DataFormat1)		; 05 - no decimal places, value is BCD
		de	0				; 06 - setting 1
		de	0				; 07 - setting 2

		de	0				; 08 - Sensor 2 flags
		de	1				; 09 - Sensor 2 ID
		de	tc_tMotor1			; 0A - Sensor 2 Class (Motor 1 temperature)
		de	0				; 0B - Sensor 2 requests per second
		de	ot_TempF			; 0C - Sensor 2 displays as temperature
		de	(1<<of_DataFormat1)|(1<<of_Dec1) ; 0D - 1 decimal place, value is BCD, use period as decimal place
		de	0				; 0E - setting 1
		de	0				; 0F - setting 2

		ORG	0x2180

Vendor1Start	de	(Vendor1End-Vendor1Start)-1
		de	0x01				; manufacturer ID
		de	"Motor RPM sensor "
		de	"Copyright (C) 2008 By XPS"
Vendor1End

		ORG	0x21C0


Vendor2Start	de	(Vendor2End-Vendor2Start)-1
		de	0x01				; manufacturer ID
		de	"Motor temperature sensor "
		de	"Copyright (C) 2008 By XPS"
Vendor2End



		END					; directive 'end of program'
